==> shuttle-0.36.0.tar.gz: shuttle-0.36.0/deployer/src/proxy.rs <==
use std::{
    convert::Infallible,
    net::{IpAddr, SocketAddr},
};

use async_trait::async_trait;
use axum::headers::HeaderMapExt;
use fqdn::FQDN;
use hyper::{
    client::{connect::dns::GaiResolver, HttpConnector},
    header::{HeaderValue, HOST, SERVER},
    Body, Client, Request, Response, StatusCode,
};
use hyper_reverse_proxy::{ProxyError, ReverseProxy};
use once_cell::sync::Lazy;
use opentelemetry::global;
use opentelemetry_http::HeaderExtractor;
use shuttle_common::backends::headers::XShuttleProject;
use tracing::{error, field, instrument, trace, Span};
use tracing_opentelemetry::OpenTelemetrySpanExt;

static PROXY_CLIENT: Lazy<ReverseProxy<HttpConnector<GaiResolver>>> =
    Lazy::new(|| ReverseProxy::new(Client::new()));
static SERVER_HEADER: Lazy<HeaderValue> = Lazy::new(|| "shuttle.rs".parse().unwrap());

#[instrument(name = "proxy_request", skip_all, fields(http.method = %req.method(), http.uri = %req.uri(), http.status_code = field::Empty, http.host = field::Empty, shuttle.service.name = field::Empty, proxy.status_code = field::Empty))]
pub async fn handle(
    remote_address: SocketAddr,
    fqdn: FQDN,
    req: Request<Body>,
    address_getter: impl AddressGetter,
) -> Result<Response<Body>, Infallible> {
    let span = Span::current();
    let parent_context = global::get_text_map_propagator(|propagator| {
        propagator.extract(&HeaderExtractor(req.headers()))
    });
    span.set_parent(parent_context);

    let host: FQDN = match req.headers().get(HOST) {
        Some(host) => host
            .to_str()
            .unwrap_or_default()
            .parse::<FQDN>()
            .unwrap_or_default()
            .to_owned(),
        None => {
            trace!("proxy request has no host header");
            Span::current().record("proxy.status_code", StatusCode::BAD_REQUEST.as_u16());
            return Ok(Response::builder()
                .status(StatusCode::BAD_REQUEST)
                .body(Body::empty())
                .unwrap());
        }
    };
    span.record("http.host", host.to_string());

    if host != fqdn {
        trace!(?host, "proxy won't serve foreign domain");
        Span::current().record("proxy.status_code", StatusCode::BAD_REQUEST.as_u16());
        return Ok(Response::builder()
            .status(StatusCode::BAD_REQUEST)
            .body(Body::from("this domain is not served by proxy"))
            .unwrap());
    }

    // We only have one service per project, and its name coincides
    // with that of the project
    let service = match req.headers().typed_get::<XShuttleProject>() {
        Some(project) => project.0,
        None => {
            trace!("proxy request has no X-Shuttle-Project header");
            Span::current().record("proxy.status_code", StatusCode::BAD_REQUEST.as_u16());
            return Ok(Response::builder()
                .status(StatusCode::BAD_REQUEST)
                .body(Body::from("request has no X-Shuttle-Project header"))
                .unwrap());
        }
    };

    // Record current service for tracing purposes
    span.record("shuttle.service.name", &service);

    let proxy_address = match address_getter.get_address_for_service(&service).await {
        Ok(Some(address)) => address,
        Ok(None) => {
            trace!(?host, service, "service not found on this server");
            Span::current().record("proxy.status_code", StatusCode::NOT_FOUND.as_u16());

            let response_body = format!("could not find service: {}", service);
            return Ok(Response::builder()
                .status(StatusCode::NOT_FOUND)
                .body(response_body.into())
                .unwrap());
        }
        Err(err) => {
            error!(error = %err, service, "proxy failed to find address for host");
            Span::current().record(
                "proxy.status_code",
                StatusCode::INTERNAL_SERVER_ERROR.as_u16(),
            );

            let response_body = format!("failed to find service for host: {}", host);
            return Ok(Response::builder()
                .status(StatusCode::INTERNAL_SERVER_ERROR)
                .body(response_body.into())
                .unwrap());
        }
    };

    match reverse_proxy(remote_address.ip(), &proxy_address.to_string(), req).await {
        Ok(response) => {
            Span::current().record("http.status_code", response.status().as_u16());
            Span::current().record("proxy.status_code", StatusCode::OK.as_u16());
            Ok(response)
        }
        Err(error) => {
            match error {
                ProxyError::InvalidUri(e) => {
                    error!(error = %e, "error while handling request in reverse proxy: 'invalid uri'");
                }
                ProxyError::HyperError(e) => {
                    error!(error = %e, "error while handling request in reverse proxy: 'hyper error'");
                }
                ProxyError::ForwardHeaderError => {
                    error!("error while handling request in reverse proxy: 'fwd header error'");
                }
                ProxyError::UpgradeError(e) => error!(error = %e,
                    "error while handling request needing upgrade in reverse proxy"
                ),
            };

            Span::current().record(
                "proxy.status_code",
                StatusCode::INTERNAL_SERVER_ERROR.as_u16(),
            );

            Ok(Response::builder()
                .status(StatusCode::INTERNAL_SERVER_ERROR)
                .body(Body::empty())
                .unwrap())
        }
    }
}

#[async_trait]
pub trait AddressGetter: Clone + Send + Sync + 'static {
    async fn get_address_for_service(
        &self,
        service_name: &str,
    ) -> crate::handlers::Result<Option<SocketAddr>>;
}

#[instrument(skip(req))]
async fn reverse_proxy(
    remote_ip: IpAddr,
    service_address: &str,
    req: Request<Body>,
) -> Result<Response<Body>, ProxyError> {
    let forward_uri = format!("http://{service_address}");
    let mut response = PROXY_CLIENT.call(remote_ip, &forward_uri, req).await?;

    response.headers_mut().insert(SERVER, SERVER_HEADER.clone());

    Ok(response)
}

==> shuttle-0.38.0.tar.gz: shuttle-0.38.0/deployer/src/proxy.rs <==
use std::{
    convert::Infallible,
    net::{IpAddr, SocketAddr},
};

use async_trait::async_trait;
use axum::headers::HeaderMapExt;
use fqdn::FQDN;
use hyper::{
    client::{connect::dns::GaiResolver, HttpConnector},
    header::{HeaderValue, HOST, SERVER},
    Body, Client, Request, Response, StatusCode,
};
use hyper_reverse_proxy::{ProxyError, ReverseProxy};
use once_cell::sync::Lazy;
use opentelemetry::global;
use opentelemetry_http::HeaderExtractor;
use shuttle_common::backends::headers::XShuttleProject;
use tracing::{error, field, instrument, trace, Span};
use tracing_opentelemetry::OpenTelemetrySpanExt;

static PROXY_CLIENT: Lazy<ReverseProxy<HttpConnector<GaiResolver>>> =
    Lazy::new(|| ReverseProxy::new(Client::new()));
static SERVER_HEADER: Lazy<HeaderValue> = Lazy::new(|| "shuttle.rs".parse().unwrap());

#[instrument(name = "proxy_request", skip_all, fields(http.method = %req.method(), http.uri = %req.uri(), http.status_code = field::Empty, http.host = field::Empty, shuttle.service.name = field::Empty, shuttle.project.name = field::Empty, proxy.status_code = field::Empty))]
pub async fn handle(
    remote_address: SocketAddr,
    fqdn: FQDN,
    req: Request<Body>,
    address_getter: impl AddressGetter,
) -> Result<Response<Body>, Infallible> {
    let span = Span::current();
    let parent_context = global::get_text_map_propagator(|propagator| {
        propagator.extract(&HeaderExtractor(req.headers()))
    });
    span.set_parent(parent_context);

    let host: FQDN = match req.headers().get(HOST) {
        Some(host) => host
            .to_str()
            .unwrap_or_default()
            .parse::<FQDN>()
            .unwrap_or_default()
            .to_owned(),
        None => {
            trace!("proxy request has no host header");
            Span::current().record("proxy.status_code", StatusCode::BAD_REQUEST.as_u16());
            return Ok(Response::builder()
                .status(StatusCode::BAD_REQUEST)
                .body(Body::empty())
                .unwrap());
        }
    };
    span.record("http.host", host.to_string());

    if host != fqdn {
        trace!(?host, "proxy won't serve foreign domain");
        Span::current().record("proxy.status_code", StatusCode::BAD_REQUEST.as_u16());
        return Ok(Response::builder()
            .status(StatusCode::BAD_REQUEST)
            .body(Body::from("this domain is not served by proxy"))
            .unwrap());
    }

    // We only have one service per project, and its name coincides
    // with that of the project
    let service = match req.headers().typed_get::<XShuttleProject>() {
        Some(project) => project.0,
        None => {
            trace!("proxy request has no X-Shuttle-Project header");
            Span::current().record("proxy.status_code", StatusCode::BAD_REQUEST.as_u16());
            return Ok(Response::builder()
                .status(StatusCode::BAD_REQUEST)
                .body(Body::from("request has no X-Shuttle-Project header"))
                .unwrap());
        }
    };

    // Record current service for tracing purposes
    span.record("shuttle.project.name", &service);
    span.record("shuttle.service.name", &service);

    let proxy_address = match address_getter.get_address_for_service(&service).await {
        Ok(Some(address)) => address,
        Ok(None) => {
            trace!(?host, service, "service not found on this server");
            Span::current().record("proxy.status_code", StatusCode::NOT_FOUND.as_u16());

            let response_body = format!("could not find service: {}", service);
            return Ok(Response::builder()
                .status(StatusCode::NOT_FOUND)
                .body(response_body.into())
                .unwrap());
        }
        Err(err) => {
            error!(error = %err, service, "proxy failed to find address for host");
            Span::current().record(
                "proxy.status_code",
                StatusCode::INTERNAL_SERVER_ERROR.as_u16(),
            );

            let response_body = format!("failed to find service for host: {}", host);
            return Ok(Response::builder()
                .status(StatusCode::INTERNAL_SERVER_ERROR)
                .body(response_body.into())
                .unwrap());
        }
    };

    match reverse_proxy(remote_address.ip(), &proxy_address.to_string(), req).await {
        Ok(response) => {
            Span::current().record("http.status_code", response.status().as_u16());
            Span::current().record("proxy.status_code", StatusCode::OK.as_u16());
            Ok(response)
        }
        Err(error) => {
            match error {
                ProxyError::InvalidUri(e) => {
                    error!(error = %e, "error while handling request in reverse proxy: 'invalid uri'");
                }
                ProxyError::HyperError(e) => {
                    error!(error = %e, "error while handling request in reverse proxy: 'hyper error'");
                }
                ProxyError::ForwardHeaderError => {
                    error!("error while handling request in reverse proxy: 'fwd header error'");
                }
                ProxyError::UpgradeError(e) => error!(error = %e,
                    "error while handling request needing upgrade in reverse proxy"
                ),
            };

            Span::current().record(
                "proxy.status_code",
                StatusCode::INTERNAL_SERVER_ERROR.as_u16(),
            );

            Ok(Response::builder()
                .status(StatusCode::INTERNAL_SERVER_ERROR)
                .body(Body::empty())
                .unwrap())
        }
    }
}

#[async_trait]
pub trait AddressGetter: Clone + Send + Sync + 'static {
    async fn get_address_for_service(
        &self,
        service_name: &str,
    ) -> crate::handlers::Result<Option<SocketAddr>>;
}

#[instrument(skip(req))]
async fn reverse_proxy(
    remote_ip: IpAddr,
    service_address: &str,
    req: Request<Body>,
) -> Result<Response<Body>, ProxyError> {
    let forward_uri = format!("http://{service_address}");
    let mut response = PROXY_CLIENT.call(remote_ip, &forward_uri, req).await?;

    response.headers_mut().insert(SERVER, SERVER_HEADER.clone());

    Ok(response)
}
